/*
 * Pin Tutorial IISWC-07
 * Microarchitecture-Independent Workload Characterization Studies Using Pin
 * 
 * Kenneth Hoste & Lieven Eeckhout
 * kehoste@elis.ugent.be, leeckhou@elis.ugent.be
 *
 * more info, see http://www.elis.ugent.be/~kehoste/mica
 *
 * Example 1: naive instruction count
 */

#include "pin.H"
#include <stdlib.h>
#include <stdio.h>


/* processing */

INT64 instr_count;

VOID instr(){

	// Every time a static instruction is executed, increase counter
	// So, we are computing the dynamic instruction count

	instr_count++;
}

/* instrumentation */
VOID Instruction(INS ins, VOID *v){

	// Insert function instr() before every static instruction

	INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR) instr, IARG_END);
}

/* output */
VOID Fini(INT32 code, VOID *v){

	fprintf(stderr,"number of instructions executed: %lld\n", instr_count);

}

int main(int argc, char * argv[]){

	instr_count = 0;

	PIN_Init(argc, argv); // initialize

	INS_AddInstrumentFunction(Instruction, 0); // instrumentation function

	PIN_AddFiniFunction(Fini, 0); // exit function

	PIN_StartProgram(); // Start the program (never returns!)
}
