/*
 * Pin Tutorial IISWC-07
 * Microarchitecture-Independent Workload Characterization Studies Using Pin
 * 
 * Kenneth Hoste & Lieven Eeckhout
 * kehoste@elis.ugent.be, leeckhou@elis.ugent.be
 *
 * more info, see http://www.elis.ugent.be/~kehoste/mica
 *
 * Example 2: determine instruction type
 */

#include "pin.H"
#include <stdlib.h>
#include <stdio.h>

/* processing */

INT64 cond_branches, muls;

VOID condBr_ins(){ cond_branches++; }
VOID mul_ins(){ muls++; }

/* instrumentation */
VOID Instruction(INS ins, VOID *v){

	char cat[50]; char opcode[50];

	strcpy(cat, CATEGORY_StringShort(INS_Category(ins)).c_str());
	strcpy(opcode, INS_Mnemonic(ins).c_str());

	if(strcmp(cat,"COND_BR") == 0) 
		INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)condBr_ins, IARG_END);

	if(strcmp(opcode,"MUL") == 0)
		INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)mul_ins, IARG_END);
}

/* output */

VOID Fini(INT32 code, VOID *v){
	fprintf(stderr,"%lld cond. branches, %lld muls\n", cond_branches, muls);
}

int main(int argc, char * argv[]){

	cond_branches = 0;
	muls = 0;

	PIN_Init(argc, argv); // initialize

	INS_AddInstrumentFunction(Instruction, 0); // instrumentation function

	PIN_AddFiniFunction(Fini, 0); // exit function

	PIN_StartProgram(); // Start the program (never returns!)
}
