/*
 * Pin Tutorial IISWC-07
 * Microarchitecture-Independent Workload Characterization Studies Using Pin
 * 
 * Kenneth Hoste & Lieven Eeckhout
 * kehoste@elis.ugent.be, leeckhou@elis.ugent.be
 *
 * more info, see http://www.elis.ugent.be/~kehoste/mica
 *
 * Example 3: determine branch IDs, the _WRONG_ way
 */

#include "pin.H"
#include <stdlib.h>
#include <stdio.h>

/* processing */

#define MAX_BR_CNT 10000
UINT32 br_cnt; INT64* branch;

VOID br(UINT32 id){ branch[id]++; }

/* instrumentation */
VOID Instruction(INS ins, VOID *v){

	char cat[50]; strcpy(cat, CATEGORY_StringShort(INS_Category(ins)).c_str());

	if(strcmp(cat,"COND_BR") == 0){

		INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)br, IARG_UINT32, br_cnt, IARG_END);

		br_cnt++;
	}
}

/* output */
VOID Fini(INT32 code, VOID *v){ UINT32 i;
	for(i=0; i < br_cnt; i++) { fprintf(stderr,"branch %d: %lld\n", i, branch[i]); }
}

int main(int argc, char * argv[]){

	INT32 i;

	br_cnt = 0;
	branch = (INT64*)malloc(MAX_BR_CNT*sizeof(INT64));
	for(i=0; i < MAX_BR_CNT; i++)
		branch[i] = 0;

	PIN_Init(argc, argv); // initialize

	INS_AddInstrumentFunction(Instruction, 0); // instrumentation function

	PIN_AddFiniFunction(Fini, 0); // exit function

	PIN_StartProgram(); // Start the program (never returns!)
}
