/*
 * Pin Tutorial IISWC-07
 * Microarchitecture-Independent Workload Characterization Studies Using Pin
 * 
 * Kenneth Hoste & Lieven Eeckhout
 * kehoste@elis.ugent.be, leeckhou@elis.ugent.be
 *
 * more info, see http://www.elis.ugent.be/~kehoste/mica
 *
 * Example 5: determine branch IDs, the correct way, and fast!
 */

#include "pin.H"
#include <stdlib.h>
#include <stdio.h>

/* processing */

UINT32 br_cnt; INT64* branch;

#define MAX_BR_CNT 10000
ADDRINT* map;

UINT32 lookup(ADDRINT a) { 

	// find index for instr. address or create new 
	UINT32 i;

	for(i=0; i < br_cnt; i++){
		if(map[i] == a)
			return i;
	}
	if(i < MAX_BR_CNT){
		map[i] = a;
		br_cnt++;
		return i; 
	}
	else{
		fprintf(stderr,"I'm sorry, I'm only allowed to support %lld static conditional branches!\n", (INT64)MAX_BR_CNT);
		exit(1);
	}
}

VOID br(UINT32 id){ branch[id]++; }

/* instrumentation */
VOID Instruction(INS ins, VOID *v){

	UINT32 id;

	char cat[50]; strcpy(cat, CATEGORY_StringShort(INS_Category(ins)).c_str());

	if(strcmp(cat,"COND_BR") == 0){

		id = lookup(INS_Address(ins));

		INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)br, IARG_UINT32, id, IARG_END);
	}
}

/* output */
VOID Fini(INT32 code, VOID *v){ UINT32 i;
	for(i=0; i < br_cnt; i++) { fprintf(stderr,"branch %d: %lld\n", i, branch[i]); }
}

int main(int argc, char * argv[]){

	INT32 i;

	br_cnt = 1;
	branch = (INT64*) malloc(MAX_BR_CNT*sizeof(INT64));
	for(i=0; i < MAX_BR_CNT; i++)
		branch[i] = 0;
	map = (ADDRINT*) malloc(MAX_BR_CNT*sizeof(ADDRINT));
	for(i=0; i < MAX_BR_CNT; i++)
		map[i] = 0;

	PIN_Init(argc, argv); // initialize

	INS_AddInstrumentFunction(Instruction, 0); // instrumentation function

	PIN_AddFiniFunction(Fini, 0); // exit function

	PIN_StartProgram(); // Start the program (never returns!)
}
