/*
 * Pin Tutorial IISWC-07
 * Microarchitecture-Independent Workload Characterization Studies Using Pin
 * 
 * Kenneth Hoste & Lieven Eeckhout
 * kehoste@elis.ugent.be, leeckhou@elis.ugent.be
 *
 * more info, see http://www.elis.ugent.be/~kehoste/mica
 *
 * Example 6: determine branch taken/not-taken rate
 */

#include "pin.H"
#include <stdlib.h>
#include <stdio.h>

/* processing */

ADDRINT ba; INT64 brCnt, t, nt; BOOL last_br;

VOID br(ADDRINT na){ brCnt++; ba = na; last_br = true; }

VOID instr(ADDRINT a) {
 
	if(last_br) { 
		if (a != ba) { 
			t++; 
		} 
		else { 
			nt++; 
		} 
		last_br = false;
	} 
}

/* instrumentation */
VOID Instruction(INS ins, VOID *v){

	char cat[50]; strcpy(cat, CATEGORY_StringShort(INS_Category(ins)).c_str());
	
	INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)instr, IARG_ADDRINT, INS_Address(ins), IARG_END);

	if(strcmp(cat,"COND_BR") == 0) 
		INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)br, IARG_ADDRINT, INS_NextAddress(ins), IARG_END);
}

/* output */
VOID Fini(INT32 code, VOID *v){
	fprintf(stderr,"%lf taken, %lf not taken\n", (double)t/brCnt, (double)nt/brCnt);
}

int main(int argc, char * argv[]){

	brCnt = 0;
	t = 0; nt = 0;
	last_br = false;
	ba = 0;

	PIN_Init(argc, argv); // initialize

	INS_AddInstrumentFunction(Instruction, 0); // instrumentation function

	PIN_AddFiniFunction(Fini, 0); // exit function

	PIN_StartProgram(); // Start the program (never returns!)
}
