/*
 * Pin Tutorial IISWC-07
 * Microarchitecture-Independent Workload Characterization Studies Using Pin
 * 
 * Kenneth Hoste & Lieven Eeckhout
 * kehoste@elis.ugent.be, leeckhou@elis.ugent.be
 *
 * more info, see http://www.elis.ugent.be/~kehoste/mica
 *
 * Example 7: determine number of register reads
 */

#include "pin.H"
#include <stdlib.h>
#include <stdio.h>

/* processing */

INT64 reg_reads;
INT64 reg_writes;

VOID regRead() { reg_reads++; }
VOID regWrite() { reg_writes++; }

/* instrumentation */
VOID Instruction(INS ins, VOID *v){

	INT32 i; INT32 max = INS_MaxNumRRegs(ins);

	for(i=0; i < max; i++) { 

		const REG reg = INS_RegR(ins, i);

		if( REG_valid(reg)){ // ALL registers (segment, fp, gen. purp., ...)

			INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)regRead, IARG_END);

		}
	} 

	max = INS_MaxNumWRegs(ins);
	for(i=0; i < max; i++) { 

		const REG reg = INS_RegW(ins, i);

		if( REG_valid(reg)){ // ALL registers (segment, fp, gen. purp., ...)

			INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)regWrite, IARG_END);

		}
	} 
}

/* output */
VOID Fini(INT32 code, VOID *v){
	fprintf(stderr,"%lld register reads\n",reg_reads);
	fprintf(stderr,"%lld register writes\n",reg_writes);
}

int main(int argc, char * argv[]){

	reg_reads = 0;
	reg_writes = 0;

	PIN_Init(argc, argv); // initialize

	INS_AddInstrumentFunction(Instruction, 0); // instrumentation function

	PIN_AddFiniFunction(Fini, 0); // exit function

	PIN_StartProgram(); // Start the program (never returns!)
}
