/*
 * Pin Tutorial IISWC-07
 * Microarchitecture-Independent Workload Characterization Studies Using Pin
 * 
 * Kenneth Hoste & Lieven Eeckhout
 * kehoste@elis.ugent.be, leeckhou@elis.ugent.be
 *
 * more info, see http://www.elis.ugent.be/~kehoste/mica
 *
 * Example 8: Determine register ids
 */

#include "pin.H"
#include <stdlib.h>
#include <stdio.h>

/* processing */
#define MAX_REG_CNT 4096
INT64* reg_reads;

VOID reg_read(UINT32 id) { reg_reads[id]++; }

/* instrumentation */
VOID Instruction(INS ins, VOID *v){

	UINT32 i; UINT32 max = INS_MaxNumRRegs(ins);

	for(i=0; i < max; i++){ 

		const REG reg = INS_RegR(ins, i);

		if( REG_valid(reg)){
			INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)reg_read, IARG_UINT32, reg, IARG_END);
		}
	} 
}

/* output */
VOID Fini(INT32 code, VOID *v){ UINT32 i;
	for (i=0; i < MAX_REG_CNT; i++) {
		if(reg_reads[i] > 0) fprintf(stderr,"[%s] %lld\n", REG_StringShort((REG)i).c_str(), reg_reads[i]); } 
}

int main(int argc, char * argv[]){

	UINT32 i;

	reg_reads = (INT64*)malloc(MAX_REG_CNT*sizeof(INT64));
	for(i=0; i < MAX_REG_CNT; i++){
		reg_reads[i] = 0;
	}

	PIN_Init(argc, argv); // initialize

	INS_AddInstrumentFunction(Instruction, 0); // instrumentation function

	PIN_AddFiniFunction(Fini, 0); // exit function

	PIN_StartProgram(); // Start the program (never returns!)
}
