/*
 * Pin Tutorial IISWC-07
 * Microarchitecture-Independent Workload Characterization Studies Using Pin
 * 
 * Kenneth Hoste & Lieven Eeckhout
 * kehoste@elis.ugent.be, leeckhou@elis.ugent.be
 *
 * more info, see http://www.elis.ugent.be/~kehoste/mica
 *
 * Example 9: determine memory reads/writes
 */

#include "pin.H"
#include <stdlib.h>
#include <stdio.h>

/* processing */
INT64 ri, wi, rs;

VOID read_ins() { ri++; rs++; } 
VOID read() { rs++; } 
VOID write_ins() { wi++; }

/* instrumentation */
VOID Instruction(INS ins, VOID *v){

	if( INS_IsMemoryRead(ins) ){

		INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)read_ins, IARG_END);

		if( INS_HasMemoryRead2(ins) )
			INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)read, IARG_END);
	}

	if( INS_IsMemoryWrite(ins) ){
		INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)write_ins, IARG_END);
	}
}

/* output */
VOID Fini(INT32 code, VOID *v){
   fprintf(stderr,"%lld load ins (%lld loads), %lld store ins", ri, rs, wi);
}

int main(int argc, char * argv[]){

	ri = 0;
	rs = 0;
	wi = 0;

	PIN_Init(argc, argv); // initialize

	INS_AddInstrumentFunction(Instruction, 0); // instrumentation function

	PIN_AddFiniFunction(Fini, 0); // exit function

	PIN_StartProgram(); // Start the program (never returns!)
}
