/*
 * Pin Tutorial IISWC-07
 * Microarchitecture-Independent Workload Characterization Studies Using Pin
 * 
 * Kenneth Hoste & Lieven Eeckhout
 * kehoste@elis.ugent.be, leeckhou@elis.ugent.be
 *
 * more info, see http://www.elis.ugent.be/~kehoste/mica
 *
 * Example 10: determine memory addresses
 */

#include "pin.H"
#include <stdlib.h>
#include <stdio.h>

/* processing */
VOID r_mem(ADDRINT a) { fprintf(stderr,"memory read @ %x\n",a); }
VOID w_mem(ADDRINT a) { fprintf(stderr,"memory write @ %x\n",a); }

/* instrumentation */
VOID Instruction(INS ins, VOID *v){

	if( INS_IsMemoryRead(ins) ){

		INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)r_mem, IARG_MEMORYREAD_EA, IARG_END);

		if( INS_HasMemoryRead2(ins) )
			INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)r_mem, IARG_MEMORYREAD2_EA, IARG_END);
	}
	if( INS_IsMemoryWrite(ins) )
		INS_InsertCall(ins, IPOINT_BEFORE, (AFUNPTR)w_mem, IARG_MEMORYWRITE_EA, IARG_END);
}

/* output */
VOID Fini(INT32 code, VOID *v){ }

int main(int argc, char * argv[]){

	PIN_Init(argc, argv); // initialize

	INS_AddInstrumentFunction(Instruction, 0); // instrumentation function

	PIN_AddFiniFunction(Fini, 0); // exit function

	PIN_StartProgram(); // Start the program (never returns!)
}
